var input;

var displayMessage = function(msg) {
	let value = input.value;
	input.value = msg;

	setTimeout(() => {
		input.value = value;
	}, 2000);
};

var insertAtCursor = function(obj, text) {
	let data = obj.value;
	let pos = obj.selectionStart;
	let start = data.slice(0, obj.selectionStart);
	let end = data.slice(obj.selectionEnd);

	obj.value = `${start}${text}${end}`;
	if (text.length > 1 && text[text.length - 1] == ')') {
		obj.selectionStart = pos + text.length - 2;
		obj.selectionEnd = pos + text.length - 1;
	} else {
		obj.selectionStart = pos + text.length;
		obj.selectionEnd = pos + text.length;
	}
};

var isKbShortcut = function(key) {
	key = key.toUpperCase();

	var shortcuts = { A: 'abs(0)', T:'tan(0)', E:'sec(0)', M: '%', P: '^', Q:'sqrt(0)', O:'cos(0)', S: 'sin(0)', L: 'log(0)', I: '3.14159' };

	if (key in shortcuts) return shortcuts[key];
	else return false;
};

var processKeyStroke = function(key) {
	//input.focus();

	if (key.length > 1 && key[key.length - 1] == ')') insertAtCursor(input, key);

	if (isKbShortcut(key)) insertAtCursor(input, isKbShortcut(key));

	if (key.match(/^([\d\^\%\*\.\+\-\/\(\)]*)$/) !== null) insertAtCursor(input, key);
	else {
		key = key.toUpperCase();
		if (key == 'C') input.value = '';
		else if (key == '=') {
			try {
				let result    = math.eval(input.value);
				let strResult = String(math.format(result, { notation: 'fixed', precision: 14 }));
				let precision = 0;

				// Wacky workaround for floating point math errors
				// -----------------------------------------------
				// ex. 12.9*3.1234=40.29186
				// returns: 40.29186000000001
				//  answer: 40.29186
				// -----------------------------------------------
				// 1. Slice off the last digit of the decimal number
				// 2. Reverse the string
				// 3. Convert the reversed number to an int 
				// 4. The precision amount is the length of the int
				// -----------------------------------------------
				let exp        = strResult.split('.')[1].slice(0, -1);
				let reverse    = exp.split("").reverse().join("");
				let stripZeros = String(parseInt(reverse));

				// Calculate the precision
				if (stripZeros != '0')
					precision = stripZeros.length < 10 ? stripZeros.length : 10;

				// Display	
				if (precision == 0)
					input.value += '=' + math.format(result, { notation: 'fixed' });
				else
					input.value += '=' + math.format(result, { notation: 'fixed', precision: precision });

			} catch (err) {
				displayMessage('Invalid Expression');
			}
		} else if (key == '<<') input.value = input.value.slice(0, -1);
	}
};

var handlePaste = function(e) {
	var clipboardData, pastedData;

	// Stop data actually being pasted into div
	e.stopPropagation();
	e.preventDefault();

	// Get pasted data via clipboard API
	clipboardData = e.clipboardData || window.clipboardData;
	pastedData = clipboardData.getData('Text');

	// Do whatever with pasteddata
	const text = (e.originalEvent || e).clipboardData.getData('text/plain');

	// Filter the pasted data
	insertAtCursor(input, text);

	// If the last char is an = then process the expression
	if (text[text.length - 1] == '=') {
		processKeyStroke('<<');
		processKeyStroke('=');
	}
};

document.addEventListener('DOMContentLoaded', function() {
	input = document.querySelector('input');

	// To past use shift-insert or the mouse
	// Ctrl-V dosn't work
	document.addEventListener('paste', handlePaste);

	document.addEventListener('keydown', x => {
		// Process these key strokes normally
		if (
			x.key == 'Home' ||
			x.key == 'ArrowRight' ||
			x.key == 'ArrowLeft' ||
			x.key == 'End' ||
			x.key == 'Backspace' ||
			x.key == 'Delete' ||
			x.key == 'Insert' ||
			x.key == 'Shift' ||
			x.key == 'Control'
		) {
			input.focus();
		} else {
			x.preventDefault();
			processKeyStroke(x.key);
		}
	});

	// Wire the events
	document.querySelectorAll('.content>div.key').forEach(e => {
		e.addEventListener('click', () => {
			processKeyStroke(e.getAttribute('data-val'));
		});
	});
});
